// Copyright 2006 and onwards
// Author: dtarlow@cs.toronto.edu (Danny Tarlow)
// 9/11/2006
//
// Interface defining graph structure functionality.

#ifndef CYBORG_BINARY_BELIEF_PROPAGATION_H__
#define CYBORG_BINARY_BELIEF_PROPAGATION_H__

using namespace std;

class BinaryFactorGraph;
class BinaryVariable;
class Message;
class MessageQueue;
class Edge;

#include <vector>
#include <string>
#include <iostream>
#include "utils.h"


class BinaryBeliefPropagation {

 public:
  BinaryBeliefPropagation(BinaryFactorGraph *fg);

  ~BinaryBeliefPropagation();

  void SetSchedule(message_schedule_t schedule) { schedule_ = schedule; }

  void InitMessagePassing();

  double PassMessages();

  double Belief(BinaryVariable *variable);

  double BeliefByGrid(string name, vector<int> &index);

  void AssignVariablesByBelief();

  void AssignVariablesByHardConstraints();

  BinaryFactorGraph *FactorGraph() { return fg_; }

  void SetVerbose(bool verbose) { verbose_ = verbose; }

 private:

  BinaryFactorGraph *fg_;

  message_schedule_t schedule_;

  bool converged_;

  int messages_sent_;

  int iteration_;

  bool verbose_;

};

#endif
