// Copyright 2009 and onwards
// Author: dtarlow@cs.toronto.edu (Danny Tarlow)
// 7/25/09

#ifndef CYBORG_BINARY_VARIABLE_H__
#define CYBORG_BINARY_VARIABLE_H__

using namespace std;

class Edge;
class Factor;

#include <string>
#include <vector>


class BinaryVariable {

 public:
  BinaryVariable(string name);

  ~BinaryVariable();

  string Name() {
    return name_;
  }

  void AddEdge(Edge *edge);

  const vector<Edge *> *Edges() { return edges_; }


  bool IsDirty();

  void Touch();

  void Touch(int position_in_variable);

  void ComputeOutgoingMessage(int position_in_variable);

  void ComputeAllOutgoingMessages();

  void ComputeAllOutgoingMessagesExcept(int position_in_variable);

  double OutgoingMessage(int position_in_variable) {
    return outgoing_messages_->at(position_in_variable);
  }

  void Neighbors(vector<Factor *> &neighbors);

  double UpdateOutgoingMessage(int position_in_variable);

  double UpdateAllOutgoingMessages();

  double UpdateAllOutgoingMessagesExcept(int position_in_variable);

  //void UpdateMessage(int position_in_edges, double value);

  double Belief();

  int NumNeighbors() { return edges_->size(); }

  void AssignValue(bool value) {
    is_assigned_ = true;
    assigned_value_ = value;
  }

  bool Assignment() { return assigned_value_; }

  bool IsAssigned() { return is_assigned_; }

  void Unassign() {
    is_assigned_ = false;
  }

 protected:
  string name_;
  vector<Edge *> *edges_;
  vector<double> *outgoing_messages_;
  double outgoing_sum_;
  bool is_dirty_;
  double damping_;

  bool is_assigned_;
  bool assigned_value_;

};

#endif
