// Copyright 2009 and onwards
// Author: dtarlow@cs.toronto.edu (Danny Tarlow)
// 7/27/09

#ifndef CYBORG_COUNTS_FACTOR_H__
#define CYBORG_COUNTS_FACTOR_H__

#include "factor.h"
#include <vector>
#include <string>


class CountsFactor : public Factor {

 public:
  CountsFactor(string name);
  ~CountsFactor();
  
  void LoadIncomingMessagesFromEdges();
  void SetIncomingMessage(int var, double val);

  void ComputeOutgoingMessage(int position_in_factor);

  void ComputeAllOutgoingMessages();
  //void UpdateAllOutgoingMessages();

  void ScalePotential(double gamma) {
    for (uint i = 0; i < potential_->size(); i++) {
      potential_->at(i) *= gamma;
    }
  }

  double Likelihood();

  void SetPotential(vector<double> &f);


 protected:
  int N_;
  vector<double> *potential_;

  // Factors keep a persistent copy of outgoing messages in a
  // form that will help efficiently compute messages in the future.
  vector<double> *forced_zeros_;
  vector<double> *forced_ones_;

  // Internal caches used for dynamic programming
  vector<double> *lr_;
  vector<double> *rl_;
  
};

#endif
