// Copyright 2009 and onwards
// Author: dtarlow@cs.toronto.edu (Danny Tarlow)
// 7/25/09

#ifndef CYBORG_EDGE_H__
#define CYBORG_EDGE_H__

#include "utils.h"

class BinaryVariable;
class Factor;
class Message;

using namespace std;


class Edge {
  friend ostream &operator<<(ostream &os, Edge &e);

 public:
  
  Edge(BinaryVariable *variable, Factor *factor) {
    endpoints_ = make_pair(variable, factor);
    to_variable_ = 0;
    to_factor_ = 0;
    position_in_factor_ = -1;

  }
  
  ~Edge() {}
  
  BinaryVariable *GetVariable() { return endpoints_.first; }
  Factor *GetFactor() { return endpoints_.second; }

  double ToVariable() { return to_variable_; }
  double ToFactor() { return to_factor_; }

  int PositionInFactor() { return position_in_factor_; }

  void SetPositionInFactor(int position) {
    position_in_factor_ = position;
  }

  int PositionInVariable() { return position_in_variable_; }

  void SetPositionInVariable(int position) {
    position_in_variable_ = position;
  }

  void SetMessage(message_direction_t direction, Message *message) { 
    if (direction == TO_VARIABLE) {
      messages_.first = message;
    } else if (direction == TO_FACTOR) {
      messages_.second = message;
    } else {
      SHOW(direction);
      CHECK(false, "Unknown direction");
    }
  }


  Message *GetMessage(message_direction_t direction) {
    if (direction == TO_VARIABLE) {
      return messages_.first;

    } else if (direction == TO_FACTOR) {
      return messages_.second;

    } else {
      SHOW(direction);
      CHECK(false, "Unknown direction");
    }
  }


  void Print();

  void SetToVariable(double to_variable) { 
    CHECK_SMALL(to_variable, "Huge incoming message value");
    to_variable_ = to_variable; 
  }

  void SetToFactor(double to_factor);

  void SetToVariableAbsoluteIndex(int index) { to_variable_absolute_index_ = index; }

  void SetToFactorAbsoluteIndex(int index) { to_factor_absolute_index_ = index; }

  int ToVariableAbsoluteIndex() { return to_variable_absolute_index_; }

  int ToFactorAbsoluteIndex() { return to_factor_absolute_index_; }


 protected:
  pair<BinaryVariable *, Factor *> endpoints_;
  pair<Message *, Message *> messages_;

  double to_variable_;
  double to_factor_;

  int position_in_factor_;
  int position_in_variable_;

  int to_factor_absolute_index_;
  int to_variable_absolute_index_;

  

};

#endif
