// Copyright 2006 and onwards
// Author: dtarlow@cs.toronto.edu (Danny Tarlow)
// 9/16/2006
//
// A class to store an n_1 x n_2 x ... x n_k hyper matrix.
// Gives log(k) access time and uses low additional storage.
//
// WARNING: elements are set and returned by copy construction, so 
// really, this should only be used for elemental types like ints
// and doubles (or pointers).

#ifndef CYBORG_HYPER_MATRIX__
#define CYBORG_HYPER_MATRIX__

#include <iostream>
#include <fstream>
#include "utils.h"
#include <vector>
#include <boost/tokenizer.hpp>

using namespace boost;

template <class Element> 
class HyperMatrix {
 public:
  HyperMatrix();

  HyperMatrix(const vector<uint> &cardinalities);

  ~HyperMatrix();

  void Set(const vector<int> &index, Element elem);

  void SetAbsoluteIndex(int absolute_index, Element elem);

  void SetConstantValue(Element elem);
  
  Element Get(const vector<int> &index) const;

  Element GetAbsoluteIndex(int absolute_index) const;

  void AbsoluteToVectorIndex(int absolute_index, vector<int> *vector_index);

  void Resize(const vector<uint> &cardinalities);

  void Print();

  // Maybe these should be made private, but we'd need an iterator
  // to encapsulate their functionality.  TODO.
  int AbsoluteIndex(const vector<int> &index) const;

  int SpaceNeeded(const vector<uint> &cardinalities) const;

  const vector<uint> *Cardinalities() { return &cardinalities_; }

  const vector<uint> *Size() { return &cardinalities_; }

  int SizeByDim(int dim) { return cardinalities_[dim]; }

  void LoadFromFile(string filename);

 protected:
  // Array that will store the raw data
  vector<Element> *elements_;

  vector<uint> cardinalities_;

  DISALLOW_COPY_CONSTRUCTORS(HyperMatrix);
};

#include "hyper_matrix.cc"

#endif  // CYBORG_HYPER_MATRIX__
