#include "message.h"
#include "message_queue.h"
#include "factor.h"
#include "binary_variable.h"



ostream &operator<<(ostream &os, Message &m) {
  if (m.direction_ == TO_VARIABLE) {
    os << m.edge_->GetFactor()->Name() << "-->" << m.edge_->GetVariable()->Name();
  } else {
    os << m.edge_->GetVariable()->Name() << "-->" << m.edge_->GetFactor()->Name();
  }

  os << " " << m.CurrentValue() << " (" << m.NewValue() << ")";
  return os;
}



int Message::PositionInVariable() { 
  return edge_->PositionInVariable(); 
}


int Message::PositionInFactor() { 
  return edge_->PositionInFactor(); 
}


string Message::Name() {
  if (direction_ == TO_VARIABLE) {
    return StringPrintf("%s_to_%s", 
			edge_->GetVariable()->Name().c_str(),
			edge_->GetFactor()->Name().c_str());
  } else {
    return StringPrintf("%s_to_%s", 
			edge_->GetFactor()->Name().c_str(),
			edge_->GetVariable()->Name().c_str());
  }
}


double Message::NewValue() {
  if (direction_ == TO_VARIABLE) {
    return edge_->GetFactor()->OutgoingMessage(edge_->PositionInFactor());
  } else if (direction_ == TO_FACTOR) {
    return edge_->GetVariable()->OutgoingMessage(edge_->PositionInVariable());
  } else {
    SHOW(direction_);
    CHECK(false, "Unknown direction");
  }
  return -1;
}


double Message::CurrentValue() {
  if (direction_ == TO_VARIABLE) {
    return edge_->ToVariable();
    
  } else if (direction_ == TO_FACTOR) {
    return edge_->ToFactor();
    
  } else {
    SHOW(direction_);
    CHECK(false, "Should never reach here");
  }
}


void Message::ComputeMessage() {
  if (direction_ == TO_VARIABLE) {
    Factor *factor = edge_->GetFactor();
    int position_in_factor = edge_->PositionInFactor();
    factor->ComputeOutgoingMessage(position_in_factor);

  } else if (direction_ == TO_FACTOR) {
    BinaryVariable *variable = edge_->GetVariable();
    int position_in_variable = edge_->PositionInVariable();
    variable->ComputeOutgoingMessage(position_in_variable);

  } else {
    SHOW(direction_);
    CHECK(false, "Unknown direction to compute message in");
  }

}


void Message::UpdateMessage() {
  Factor *factor = edge_->GetFactor();
  BinaryVariable *variable = edge_->GetVariable();
  int position_in_factor = edge_->PositionInFactor();
  int position_in_variable = edge_->PositionInVariable();

  if (direction_ == TO_VARIABLE) {
    //cout << "Updating message " << factor->Name() << "-->" << variable->Name() << endl;
	
    factor->UpdateOutgoingMessage(position_in_factor);
    variable->Touch(position_in_variable);

  } else if (direction_ == TO_FACTOR) {
    //cout << "Updating message " << variable->Name() << "-->" << factor->Name() 
    //<< endl;

    variable->UpdateOutgoingMessage(position_in_variable);
    factor->Touch(position_in_factor);

  } else {
    SHOW(direction_);
    CHECK(false, "Unknown direction to compute message in");
  }

}
