// Copyright 2009 and onwards
// Author: dtarlow@cs.toronto.edu (Danny Tarlow)
// 7/25/09

#ifndef CYBORG_MESSAGE_QUEUE_H__
#define CYBORG_MESSAGE_QUEUE_H__

#include "utils.h"
#include <stack>

using namespace std;

class Message;


class MessageQueue {

 public:
  MessageQueue(message_schedule_t schedule);

  ~MessageQueue() {}

  void AddMessage(Message *message) { AddMessage(message, true); }

  void AddMessage(Message *message, bool new_message);

  Message *TopMessage();

  void UpdateMessagePriority(Message *message);

  void SetMessagePriorityToZero(Message *message);

  void UpdateHeapAtPosition(int position_in_heap);

  bool PassMessage();  // Return true upon convergence

  bool PassAsynchronousMessage();  // Return true upon convergence

  bool PassResidualMessage();  // Return true upon convergence

  void PushMessagesThroughVariables(Message *message);

  uint ActiveMessageCount() { return message_heap_->size(); }

  void StartIteration();

  uint size() { return message_heap_nodes_->size(); }

  Message *at(int position) { return message_heap_nodes_->at(position); }

  void SetDebug(bool debug) { debug_ = debug; }

  vector<Message *> *Queue() { return message_heap_; }

  double Residual();

 protected:
  vector<Message *> *message_heap_;
  vector<Message *> *message_heap_nodes_;

  stack<Message *> *message_stack_;

  int asynch_pos_in_heap_;

  bool debug_;

  int messages_sent_;

  message_schedule_t schedule_;

  __gnu_cxx::hash_map<int, int> messages_passed_;

};

#endif
