#include "potts_factor.h"
#include "utils.h"
#include <string>
#include <vector>
#include "edge.h"
#include "binary_variable.h"


PottsFactor::PottsFactor(string name) : Factor(name) {
  edges_ = new vector<Edge *>();
  outgoing_messages_ = new vector<double>();
  is_dirty_ = true;
  x_pos_ = -1;
  y_pos_ = -1;
}


PottsFactor::~PottsFactor() {
  delete outgoing_messages_;
  delete edges_;
}


void PottsFactor::SetPotential(double potential) {
  both_on_potential_ = potential;
}


void PottsFactor::ComputeOutgoingMessage(int position_in_factor) {
  bool print_stuff = false;
  if (position_in_factor == 0) {
    // Grab the incoming message off the edge.
    double m_h1_1 = edges_->at(1)->ToFactor();
    
    // If h1 = 0, there's a bonus for choosing h0 = 0
    // If h1 = 1, there's a bonus for choosing h0 = 1.
    double m_f_h0_0 = MAX(m_h1_1, both_on_potential_);    
    double m_f_h0_1 = MAX(m_h1_1 + both_on_potential_, 0);
    
    if (print_stuff) {
      cout << "    potential: " << both_on_potential_ << endl;
      cout << "    m_f_h0_0: " << m_f_h0_0 << endl;
      cout << "    m_f_h0_1: " << m_f_h0_1 << endl;
      cout << endl;
    }

    // And we just need the message difference
    double new_message = m_f_h0_1 - m_f_h0_0;
    outgoing_messages_->at(0) = new_message;

    //cout << "(" << position_in_factor << "): " << m_h1_1 << "--> [" << Name()
    //	 << " " << both_on_potential_ << "] --> " << outgoing_messages_->at(0) << endl;

  } else if (position_in_factor == 1) {
    // Grab the incoming message off the edge.
    double m_h0_1 = edges_->at(0)->ToFactor();
    

    // If h0 = 0, there's a bonus for choosing h1 = 0
    // If h0 = 1, there's a bonus for choosing h1 = 1.
    double m_f_h1_0 = MAX(m_h0_1, both_on_potential_);
    double m_f_h1_1 = MAX(m_h0_1 + both_on_potential_, 0);
    
    if (print_stuff) {
      cout << "    potential: " << both_on_potential_ << endl;
      cout << "    m_f_h1_0: " << m_f_h1_0 << endl;
      cout << "    m_f_h1_1: " << m_f_h1_1 << endl;
      cout << endl;
    }

    // And we just need the message difference
    double new_message = m_f_h1_1 - m_f_h1_0;
    outgoing_messages_->at(1) = new_message;

    //cout << "(" << position_in_factor << "): " << m_h0_1 << "--> [" << Name()
    //	 << " " << both_on_potential_ << "] --> " << outgoing_messages_->at(1) << endl;

  } else {
    CHECK(false, "Shouldn't get here");
  }
}


void PottsFactor::ComputeAllOutgoingMessages() {
  ComputeOutgoingMessage(0);
  ComputeOutgoingMessage(1);
}


double PottsFactor::Likelihood() {
  int num_on = 0;
  for (int i = 0; i < 2; i++) {
    BinaryVariable *var = scope_->at(i);
    CHECK(var->IsAssigned(), "Variable not assigned");
    if (var->Assignment()) {
      ++num_on;
    }
  }

  if (num_on == 1) {
    return -both_on_potential_;
  } else {
    return 0;
  }
}
