#include "singleton_factor.h"
#include "utils.h"
#include <string>
#include <vector>
#include "edge.h"
#include "binary_variable.h"


SingletonFactor::SingletonFactor(string name) : Factor(name) {
  edges_ = new vector<Edge *>();
  outgoing_messages_ = new vector<double>();
  is_dirty_ = true;
  on_potential_ = 0;
}


SingletonFactor::~SingletonFactor() {
  delete outgoing_messages_;
  delete edges_;
}

void SingletonFactor::SetPotential(double potential) {
  on_potential_ = potential;

  /*
  if (outgoing_messages_->size() == 0) {
    outgoing_messages_->push_back(potential);

  } else if (outgoing_messages_->size() == 1) {
    outgoing_messages_->at(0) = potential;

  } else {
    SHOW(outgoing_messages_->size());
    CHECK(false, "Weird size for singleton factor");
  }

  CHECK_EQ(outgoing_messages_->size(), 1,
	   "Scope of singleton factor is not 1");
  */
}


void SingletonFactor::Touch() {
  // A singleton factor will send the same message no matter what,
  // so it never needs to update itself.
  is_dirty_ = true;
}


void SingletonFactor::ComputeOutgoingMessage(int position_in_factor) {
  CHECK_EQ(position_in_factor, 0, "Trying to compute non existent message");
  
  double new_message = on_potential_;
  outgoing_messages_->at(0) = new_message;
  //cout << "  compute " << Name() << " " << on_potential_ << "--> var" << endl;
}


void SingletonFactor::ComputeAllOutgoingMessages() {
  // A singleton factor will send the same message no matter what,
  // so it never needs to update itself.
  CHECK_EQ(outgoing_messages_->size(), 1,
	   "Error: scope of singleton factor is not 1");
  ComputeOutgoingMessage(0);

  //cout << "SINGLETON FACTOR " << name_ << " computing message = " 
  //<< outgoing_messages_->at(0) << endl;
}


double SingletonFactor::Likelihood() {
  BinaryVariable *var = scope_->at(0);
  CHECK(var->IsAssigned(), "Variable not assigned");
  if (var->Assignment()) {
    return on_potential_;
  } else {
    return 0;
  }
}
